from django.db import models
from django.contrib.auth.models import (AbstractBaseUser, PermissionsMixin,
                                        BaseUserManager)
from django.utils import timezone


class CustomUserManager(BaseUserManager):
    def create_user(self, email, password=None, **extra_fields):
        """
        Create and save a User with the given email and password.
        """
        if not email:
            raise ValueError(_('The Email must be set'))
        email = self.normalize_email(email)
        user = self.model(email=email, **extra_fields)
        user.set_password(password)
        user.save()
        return user

    def create_superuser(self, email, password, **extra_fields):
        """
        Create and save a SuperUser with the given email and password.
        """
        extra_fields.setdefault('is_staff', True)
        extra_fields.setdefault('is_superuser', True)
        extra_fields.setdefault('is_active', True)

        if extra_fields.get('is_staff') is not True:
            raise ValueError(_('Superuser must have is_staff=True.'))
        if extra_fields.get('is_superuser') is not True:
            raise ValueError(_('Superuser must have is_superuser=True.'))
        return self.create_user(email, password, **extra_fields)

class CommonData(models.Model):
    batch_1 = models.CharField(max_length=32, blank=True)
    batch_2 = models.CharField(max_length=32, blank=True)

    class Meta:
        abstract = True

class Region(models.Model):
    region_name = models.CharField(max_length=100, blank=True)

    class Meta:
        db_table = 'Region'

class User(AbstractBaseUser, PermissionsMixin, CommonData):
    MR = 'Mr'
    MRS = 'Mrs'
    MS = 'Ms'
    TITLE_CHOICES = ((MR, 'Mr.'), (MRS, 'Mrs.'), (MS, 'Ms.'))
    username = models.CharField(max_length=150, blank=True)
    first_name = models.CharField(max_length=100, blank=True)
    last_name = models.CharField(max_length=100, blank=True)
    title = models.CharField(max_length=3, choices=TITLE_CHOICES, default=MR)
    email = models.EmailField(
        unique=True, blank=True,
        error_messages={
            "unique": "A user with that email already exists."
        },
        db_index=True
    )
    is_staff = models.BooleanField(
        default=False,
        help_text='Designates whether the user can log into this admin site.'
    )
    is_active = models.BooleanField(
        default=True,
        help_text=(
            'Designates whether this user should be treated as active. '
            'Unselect this instead of deleting accounts.'
        ),
    )
    registration_date = models.DateTimeField(default=timezone.now)
    identity_number = models.CharField(max_length=100, unique=True)
    mobile = models.CharField(max_length=20, blank=True)
    points = models.BigIntegerField(default=0)
    card_number = models.CharField(
        max_length=13, blank=True,
        help_text='Number used to populate my qrcode and barcode.'
    )
    region = models.ForeignKey(Region, on_delete=models.SET_NULL, null=True, blank=True)

    objects = CustomUserManager()

    USERNAME_FIELD = 'email'
    REQUIRED_FIELDS = []

    class Meta:
        db_table = 'User'

    def __str__(self):
        return self.email


class UserMetaData(CommonData):
    usermeta_id = models.BigAutoField(primary_key=True)
    user = models.ForeignKey(User, on_delete=models.CASCADE)
    meta_key = models.CharField(max_length=255, blank=True)
    meta_value = models.TextField(blank=True)

    class Meta:
        db_table = 'wpnp_usermeta'


class Gift(CommonData):
    code = models.CharField(max_length=13, blank=True)
    title = models.CharField(max_length=100, blank=True)
    description = models.TextField(blank=True)
    points = models.BigIntegerField(default=0)
    amount = models.DecimalField(default=0.0, max_digits=20, decimal_places=10)
    activated = models.BooleanField(default=True)

    class Meta:
        db_table = 'TGiftMT'


class Location(CommonData):
    outlet = models.CharField(max_length=255, blank=True)

    class Meta:
        db_table = 'TLocations'


class GiftTransaction(CommonData):
    transact_id = models.BigAutoField(primary_key=True)
    gift = models.ForeignKey(Gift, on_delete=models.CASCADE)
    location = models.ForeignKey(Location, on_delete=models.CASCADE)
    created = models.DateTimeField(auto_now_add=True)
    user = models.ForeignKey(User, on_delete=models.CASCADE)
    hidden = models.BooleanField(default=True)
    expired = models.BooleanField(default=False)
    redeemed = models.BooleanField(default=False)
    voucher_ref = models.CharField(max_length=13, blank=True)

    class Meta:
        db_table = 'TGiftTransacs'


class Voucher(CommonData):
    code = models.CharField(max_length=13, blank=True)
    title = models.CharField(max_length=100, blank=True)
    description = models.TextField(blank=True)
    amount = models.DecimalField(default=0.0, max_digits=20, decimal_places=10)
    activated = models.BooleanField(default=True)

    class Meta:
        db_table = 'TMoneyVouchers'


class VoucherTransaction(CommonData):
    transact_id = models.BigAutoField(primary_key=True)
    voucher = models.ForeignKey(Voucher, on_delete=models.CASCADE)
    location = models.ForeignKey(Location, on_delete=models.CASCADE)
    created = models.DateTimeField(auto_now_add=True)
    user = models.ForeignKey(User, on_delete=models.CASCADE)
    hidden = models.BooleanField(default=True)
    expired = models.BooleanField(default=False)
    redeemed = models.BooleanField(default=False)
    voucher_ref = models.CharField(max_length=13, blank=True)

    class Meta:
        db_table = 'TMoneyVouchersTransacs'


class GiftLocation(CommonData):
    gift = models.ForeignKey(Gift, on_delete=models.CASCADE)
    location = models.ForeignKey(Location, on_delete=models.CASCADE)
    activated = models.BooleanField(default=True)

    class Meta:
        db_table = 'TGiftLocations'


class VoucherLocation(CommonData):
    voucher = models.ForeignKey(Voucher, on_delete=models.CASCADE)
    location = models.ForeignKey(Location, on_delete=models.CASCADE)
    activated = models.BooleanField(default=True)

    class Meta:
        db_table = 'TMoneyVouchersLocations'


class Transaction(CommonData):
    from_user = models.ForeignKey(User, on_delete=models.CASCADE)
    card_number = models.CharField(max_length=255, blank=True)
    points = models.BigIntegerField(default=0)
    created = models.DateTimeField(auto_now_add=True)
    description = models.TextField(blank=True)
    processed = models.BooleanField(default=False)
    transac_ref = models.CharField(max_length=13, blank=True)

    class Meta:
        db_table = 'Transaction'
